@@ -66,7 +66,6 @@ void declareOIDs(void)
     return;
 
   done=1;
-  OBJ_create(email, "Email", "Email");
   OBJC(idatcap,"idatcap");
   /*  //// test */
   OBJC(attributes,"attributes");
@@ -82,16 +81,3 @@ void declareOIDs(void)
 
 }
 
-#if 0
-extern int initEx(void);
-extern void ERR_load_AC_strings(void);
-
-void InitAC(void)
-{
-  static int i = 0;
-  if (i) return;
-  i=1;
-  (void)declareOIDs();
-  (void)initEx();
-}
-#endif
