@@ -43,11 +43,11 @@
 #define VOMS_GSISOCKETSERVER
 
 /** Include the secure socket globus definition. */
-#include "globus_gss_assist.h"
 
 #include <stdio.h>
 #include <openssl/evp.h>
 #include <openssl/x509.h>
+#include <openssl/ssl.h>
 
 #include <string>
 
@@ -73,20 +73,20 @@ class GSISocketServer
    * @param p the secure server port.
    * @param b the backlog, that is the maximum number of outstanding connection requests.
    */
-  GSISocketServer(int, int, void * = NULL, int=5, bool=true);
+  GSISocketServer(int, void * = NULL, int=5, bool=true);
   /**
    * Destructor.
    * This method must be also implemented by object subclassing server socket.
    */
-  virtual ~GSISocketServer();
+  ~GSISocketServer();
 
   /**
    * Close the connection.
    */
-  virtual void Close();
-  virtual bool Open();
-  virtual void CloseListener(void);
-  virtual void CloseListened(void);
+  void Close();
+  bool Open();
+  void CloseListener(void);
+  void CloseListened(void);
 
   /**
    * Listen for incoming connection requests.
@@ -94,35 +94,19 @@ class GSISocketServer
    * @param a a reference to the secure GSI Socket Agent sent by Client.
    * @return the GSI Socket Agent redirecting communication on a dedicated port.
    */
-  virtual bool Listen();
-  /**
-   * Redirects the GSI output.
-   * This method allows to define a logging file for GSI.
-   * @param fp a pinter to a file.
-   */ 
-  void RedirectGSIOutput(FILE *fp) { gsi_logfile = fp; }
-  void SetFlags(OM_uint32 flags);
+  bool Listen();
   void SetLogger(void *log);
   void CleanSocket();
   bool Send(const std::string &s);
   bool Receive(std::string &s);
+  bool Peek(int size, std::string &s);
   bool AcceptGSIAuthentication(void); 
   void AdjustBacklog(int b);
-  bool ReOpen(int, int, int=5, bool=true);
+  bool ReOpen(int, int=5, bool=true);
   void SetTimeout(int);
-  int  GetTimeout();
-  gss_ctx_id_t GetContext();
 
- private:
-  /**
-   * Accept the GSI Authentication.
-   * @param sock the socket for communication.
-   * @param ctx the authorization context.
-   * @return the context identifier. 
-   */
-  /** The reference to the log file. */
-  FILE *gsi_logfile;
-  int version;
+  void SetError(const std::string &g);
+  void SetErrorOpenSSL(const std::string &message);
 
 public:
   std::string    own_subject;
@@ -131,24 +115,29 @@ public:
   std::string    peer_ca;
   std::string    peer_serial;
   EVP_PKEY *own_key;
-  EVP_PKEY *peer_key;
   X509 *own_cert;
   X509 *peer_cert;
   X509 *actual_cert;
   STACK_OF(X509) *own_stack;
   STACK_OF(X509) *peer_stack;
+  SSL *ssl;
+  SSL_CTX *ctx;
+  BIO *conn;
+  void *pvd;
+  char           *cacertdir;
+  EVP_PKEY       *upkey;
+  X509           *ucert;
+  std::string error;
 
-private:
+public:
   int port;
   bool opened;
-  gss_cred_id_t credential;
-  gss_ctx_id_t  context;
   int sck;
   int backlog;
-  int newsck;
+  int newsock;
+  int timeout;
   bool newopened;
   bool mustclose;
-  OM_uint32 conflags;
   void *logh;
 };
 
