@@ -37,12 +37,13 @@
 #include <unistd.h>
 #include <time.h>
 
+#include "doio.h"
+
 struct localdata {
   char *name;
   char *dateformat;
   int maxlog;
   int fd;
-  int level;
 };
 
 static int filereopen(struct localdata *ld);
@@ -54,12 +55,9 @@ static char *translate(char *format, char *date)
   char *newstring = NULL;
 
   while (position) {
-    newstring = malloc(strlen(format) + strlen(date) + 1 - 2);
     *position++='\0';
     position++;
-    newstring = strcpy(newstring, format);
-    newstring = strcat(newstring, date);
-    newstring = strcat(newstring, position);
+    newstring = snprintf_wrap("%s%s%s", format, date, position);
     free(format);
     format = newstring;
     position = strstr(format, "%d");
@@ -84,12 +82,10 @@ static int fileoutputter(void *data, const char *s)
 
   if (ld->maxlog) {
     if (position > ld->maxlog) {
-      if (logfile_rotate(ld->name)) {
-        if (!filereopen(ld))
-          write(ld->fd, "VOMS: LOGGING ROTATION ERROR\n", 29);
+      if (!logfile_rotate(ld->name) || !filereopen(ld)) {
+        UNUSED(int ret);
+        ret= write(ld->fd, "VOMS: LOGGING ROTATION ERROR\n", 29);
       }
-      else
-        write(ld->fd, "VOMS: LOGGING ROTATION ERROR\n", 29);
     }
   }
   output = strdup(s);
@@ -107,8 +103,10 @@ static int fileoutputter(void *data, const char *s)
 
     do {
       free(data);
+
       if ((data = malloc(datasize)))
         len = strftime(data, datasize, ld->dateformat, ti);
+
       datasize += 50;
     } while (len == 0 && data);
 
@@ -137,6 +135,7 @@ static int filereopen(struct localdata *ld)
     ld->fd = newfd;
     return 1;
   }
+
   return 0;
 }
 
@@ -149,6 +148,7 @@ static void filedestroy(void *data)
   
   if (ld->fd != -1)
     close(ld->fd);
+
   free (ld->name);
   free(ld);
 }
@@ -164,7 +164,6 @@ static void *fileinit(void)
     ld->dateformat = NULL;
     ld->fd       = -1;
     ld->maxlog   = 0;
-    ld->level    = -1;
   }
 
   return ld;
@@ -177,9 +176,7 @@ static void fileoptioner(void *data, const char *name, const char *value)
   if (!ld)
     return;
 
-  if (strcmp(name, "LEVEL") == 0)
-    ld->level=atoi(value);
-  else if (strcmp(name, "NAME") == 0) {
+  if (strcmp(name, "NAME") == 0) {
     int fd = open(value, O_WRONLY|O_CREAT|O_APPEND, S_IRUSR|S_IWUSR);
 
     if (fd != -1) {
@@ -215,106 +212,91 @@ void *FILEStreamerAdd(void *h)
  
 static int logfile_rotate(const char * name)
 {
-  int i = 0;
   char *pos, *dirname, *newname, *oldname, *basename;
-  int max = 0;
   DIR * dir = NULL;
   struct dirent * de = NULL;
   int result = 0;
   char *fname = NULL;
-  int res = 1;
   int fd;
 
-  pos = dirname = fname = newname = oldname = NULL;
+  newname = NULL;
 
-/*   // get the name of the directory and of the file */
+  /* get the name of the directory and of the file */
 
-  newname = malloc(strlen(name)+26);
-  oldname = malloc(strlen(name)+26);
-  fname   = malloc(strlen(name)+5);
-  dirname = malloc(strlen(name)+2);
+  fname   = snprintf_wrap("%s-lck", name);
 
-  if (!fname || !newname || !oldname || !dirname)
+  if (!fname)
     goto err;
 
-  strcpy(fname, name);
-  strcat(fname, "-lck");
-
   if ((fd = open(fname, O_CREAT|O_EXCL|O_RDONLY, S_IRUSR|S_IWUSR)) != -1) {
+    int i = 0;
+    int max = 0;
+
     pos = strrchr(name, '/');
   
     if (pos == NULL) {
-      dirname[0] = '.';
-      dirname[1] = '\0';
-      basename = pos;
+      dirname = snprintf_wrap(".");
+      basename = name;
     }
     else if (pos == name) {
-      dirname[0]='/';
-      dirname[1]='\0';
+      dirname = snprintf_wrap("/");
       basename = ++pos;
     }
     else {
-      strncpy(dirname, name, pos - name);
+      dirname = snprintf_wrap("%s", name);
       dirname[pos-name] = '\0';
       basename = ++pos;
     }
 
+    if (!dirname)
+      goto err;
+
     dir = opendir(dirname);
     if (dir) {
+      int baselen = strlen(basename);
+
       while ((de = readdir(dir))) {
         pos = strrchr(de->d_name, '.');
-        if (pos && 
-            pos - de->d_name == strlen(basename) &&
-            strncmp(basename, de->d_name, strlen(basename)) == 0 &&
-            atoi((++pos)) > max)
-          max = atoi(pos);
-      }
-    }
-    closedir(dir);
-    
 
-/*     // rename each file increasing the suffix */
-  
+        if (pos && atoi(pos+1) > max &&
+            (size_t)(pos - de->d_name) == baselen &&
+            strncmp(basename, de->d_name, baselen) == 0)
+          max = atoi(pos+1);
+      }
+      closedir(dir);
+    }
+    free(dirname);
+
+    /* rename each file increasing the suffix */
     if (max) {
       for(i = max; i > 0 ; --i) {
-        char s[24];
-        
-        strcpy(newname, name);
-        strcat(newname, ".");
-        sprintf(s, "%d", i+1);
-        strcat(newname, s);
-
-        strcpy(oldname, name);
-        strcat(oldname, ".");
-        sprintf(s, "%d", i);
-        strcat(oldname, s);
+        newname = snprintf_wrap("%s.%d", name, i+1);
+        oldname = snprintf_wrap("%s.%d", name, i);
     
-        (void)rename(oldname, newname);
+        if (newname && oldname)
+          (void)rename(oldname, newname);
+
+        free(oldname);
+        free(newname);
       }
     }
-/*     // rename the main file to .1  */
 
-    newname = (char *)malloc((strlen(name) + 3) * sizeof(char));
+    newname = snprintf_wrap("%s.1", name);
+
+    /* rename the main file to .1  */
     if (newname) {
-      strcpy(newname, name);
-      strcat(newname, ".1");
-      if (rename(name, newname) == -1)
-        res = 0;
-      result = 1;
+      if (rename(name, newname) != -1)
+        result = 1;
     }
-    
+
+    free(newname);
+
     unlink(fname);
     close(fd);
   }
 
  err:
-  free(dirname);
   free(fname);
-  free(newname);
-  free(oldname);
 
-  if (result && res)
-    return 1;
-  else
-    return 0;
+  return result;
 }
